/**
	10式画像処理機ビットマップパネルコンポーネント

	2010-06-16 創作プログラミングの街
*/
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class BMPanel extends JPanel implements MouseMotionListener {

	private JIntFrameBuffer _frame;

	private BufferedImage _img;

	private int _viewWidth, _viewHeight, _imgWidth, _imgHeight;
	private int _trimX, _trimY, _trimWidth, _trimHeight;

	private gp10j _owner;

	private double _vsSize;

	public BMPanel(gp10j arg) {

		_owner = arg;
		_frame = new JIntFrameBuffer(128, 128);

		_vsSize = 1.0;

		addMouseMotionListener(this);

	}

	public boolean setViewSize(double vs) {

		if (vs < 0.1) {
			return false;
		}

		_vsSize = vs;

		_viewWidth = (int)(_imgWidth * _vsSize);
		_viewHeight = (int)(_imgHeight * _vsSize);

		setSize(new Dimension(_viewWidth + 2, _viewHeight + 2));

		repaint();

		return true;

	}

	public void setFrameBuffer(JIntFrameBuffer frm) {

		_frame.assign(frm);

		setImageSize(_frame.getWidth(), _frame.getHeight());

		updateImage();
		repaint();

	}

	private void setImageSize(int width,int height) {

		_imgWidth = width;
		_imgHeight = height;

		// BufferedImage再確保
		if (_img == null || _img.getWidth() != width || _img.getHeight() != height) {

			_img = null;

			System.gc();
			System.runFinalization();
			System.gc();

			_img = new BufferedImage(_imgWidth, _imgHeight, BufferedImage.TYPE_INT_RGB);

		}

		_viewWidth = (int)(_imgWidth * _vsSize);
		_viewHeight = (int)(_imgHeight * _vsSize);

		setSize(new Dimension(_viewWidth + 2, _viewHeight + 2));

	}

	void updateImage() {

		// BufferedImageのピクセル列設定
		_img.getRaster().setDataElements(0, 0, _imgWidth, _imgHeight, _frame.getBuffer());

	}

	public void paintComponent(Graphics g) {

		g.setColor(getBackground());
		g.fillRect(0, 0, getWidth(), getHeight());

		if (_vsSize == 1) {
			g.drawImage(_img, 1, 1, this);
		} else {
			g.drawImage(_img, 1, 1, _viewWidth, _viewHeight, this);
		}

		if (_trimWidth > 0 && _trimHeight > 0 && _trimX >= 0 && _trimX < (_imgWidth - 1) && _trimY >= 0 && _trimY <( _imgHeight - 1)) {

			int trimX = (int)(_trimX * _vsSize);
			int trimY = (int)(_trimY * _vsSize);
			int trimWidth = 1 + (int)(_trimWidth * _vsSize);
			int trimHeight = 1 + (int)(_trimHeight * _vsSize);

			if (trimX + trimWidth > (_viewWidth + 1)) {
				trimWidth = (_viewWidth + 1) - trimX;
			}

			if (trimY + trimHeight > (_viewHeight + 1)) {
				trimHeight = (_viewHeight + 1) - trimY;
			}

			g.setColor(Color.RED);
			g.drawRect(1 + (trimX - 1), 1 + (trimY - 1), trimWidth, trimHeight);

		}

	}

	public Dimension getPreferredSize() {

		return new Dimension(_viewWidth + 2, _viewHeight + 2);

	}

	public void mouseDragged(MouseEvent e) {}

	// マウスポインタ移動
	public void mouseMoved(MouseEvent e) {

		if (e.getX() < 1 || e.getY() < 1) {
			return;
		}

		int px = (int)(e.getX() * (1.0 / _vsSize)) - 1;
		int py = (int)(e.getY() * (1.0 / _vsSize)) - 1;

		if (px >= _imgWidth || py >= _imgHeight) {
			return;
		}

		// マウス座標のRGB取得
		int r = _frame.getR(px, py);
		int g = _frame.getG(px, py);
		int b = _frame.getB(px, py);

		// マウス座標のRGB情報を文字列化
		String mes=" (" + px + "," + py + ")" + ":";
		mes = mes + r + "," + g + "," + b;

		// 文字列化したRGB情報をアプリケーションに表示
		_owner.setStatus(mes);

	}

	public void setTrimArea(int x, int y, int width, int height) {

		_trimX = x;
		_trimY = y;

		_trimWidth = width;
		_trimHeight = height;

	}

}
