import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class GammaCondDialog extends JDialog implements ActionListener, ChangeListener, ComponentListener {

	private gp10j _owner;

	private PixelsViewer _viewO, _viewP;
	private JLabel _viewLabel;

	private Box _views, _fields, _buttons;

	private JLabel _label;
	private JSlider _slider;

	private JButton _btnView, _btnExec, _btnCancel;

	private int _gamma;

	private int[] _original;
	private JIntFrameBuffer _preview;

	public GammaCondDialog(gp10j owner, JIntFrameBuffer buf) {

		super(owner, "ガンマ補正", true);

		_owner = owner;

		int w = buf.getWidth();
		int h = buf.getHeight();

		int width = 0;
		int height = 0;

		if (w > h) {

			if (w < gp10j.PREVIEW_SIZE) {
				width = w;
			} else {
				width = gp10j.PREVIEW_SIZE;
			}

			height = (h * width) / w;

		} else {

			if (h < gp10j.PREVIEW_SIZE) {
				height = h;
			} else {
				height = gp10j.PREVIEW_SIZE;
			}

			width = (w * height) / h;

		}

		_views = Box.createHorizontalBox();

		_original = buf.getScaledPixels(width, height);
		_preview = new JIntFrameBuffer(_original);

		_viewO = new PixelsViewer(_original);
		_viewLabel = new JLabel("→");
		_viewP = new PixelsViewer(_original);

		_views.add(Box.createVerticalGlue());
		_views.add(_viewO);
		_views.add(Box.createHorizontalStrut(2));
		_views.add(_viewLabel);
		_views.add(Box.createHorizontalStrut(2));
		_views.add(_viewP);
		_views.add(Box.createVerticalGlue());

		_fields = Box.createHorizontalBox();

		_label = new JLabel("ガンマ補正:1.00");
		_slider = new JSlider(0, 200, 100);
		_slider.addChangeListener(this);

		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(_label);
		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(_slider);

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_views);
		c.add(Box.createVerticalStrut(4));
		c.add(_fields);
		c.add(Box.createVerticalStrut(4));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_gamma = -1;

		addComponentListener(this);

		repaint();

	}

	public int getGamma() {
		return _gamma;
	}

	public void actionPerformed(ActionEvent e) {

		// 「実行」ボタン
		if (e.getSource() == _btnExec) {

			_gamma = _slider.getValue();

			hide();

		}

		// 「中止」ボタン
		if (e.getSource() == _btnCancel) {

			_gamma = -1;

			hide();

		}

	}

	public void stateChanged(ChangeEvent e) {

		if (e.getSource() == _slider) {

			int val = _slider.getValue();

			_label.setText("ガンマ補正:" + String.valueOf(val / 100.0));

			_preview.assign(_original);

			_preview.gamma((double)val / 100.0);

			_viewP.setPixels(_preview.getPixels());

			repaint();

		}

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
