import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class HSBCondDialog extends JDialog implements ActionListener, ChangeListener, ComponentListener {

	private gp10j _owner;

	private Box _views, _Hfields, _Sfields, _Bfields, _buttons;

	private PixelsViewer _viewO, _viewP;
	private JLabel _viewLabel;
	private JLabel _Hlabel, _Slabel, _Blabel;
	private JLabel _Hhint, _Shint, _Bhint;
	private JSlider _Hslider, _Sslider, _Bslider;

	private JButton _btnExec, _btnCancel;

	private int _h, _s, _b;

	private int[] _original;
	private JIntFrameBuffer _preview;

	public HSBCondDialog(gp10j owner, JIntFrameBuffer buf) {

		super(owner, "HSB成分調整", true);

		_owner = owner;
		addComponentListener(this);

		int w = buf.getWidth();
		int h = buf.getHeight();

		int width = 0;
		int height = 0;

		// プレビュー画像のサイズ算出
		if (w > h) {

			if (w < gp10j.PREVIEW_SIZE) {
				width = w;
			} else {
				width = gp10j.PREVIEW_SIZE;
			}

			height = (h * width) / w;

		} else {

			if (h < gp10j.PREVIEW_SIZE) {
				height = h;
			} else {
				height = gp10j.PREVIEW_SIZE;
			}

			width = (w * height) / h;

		}

		// プレビュー画像のピクセル列作成
		_original = buf.getScaledPixels(width, height);

		// プレビュー画像作成
		_preview = new JIntFrameBuffer(_original);

		_views = Box.createHorizontalBox();

		_viewO = new PixelsViewer(_original);
		_viewLabel = new JLabel("→");
		_viewP = new PixelsViewer(_original);

		_views.add(_viewO);
		_views.add(_viewLabel);
		_views.add(_viewP);

		_Hfields = Box.createHorizontalBox();

		_Hlabel = new JLabel("色相:0   ");
		_Hslider = new JSlider(-128, 128, 0);
		_Hslider.addChangeListener(this);

		_Hhint = new JLabel("?");
		_Hhint.setToolTipText("各ピクセルの色相に加算する値を設定");

		_Hfields.add(Box.createHorizontalStrut(4));
		_Hfields.add(_Hlabel);
		_Hfields.add(Box.createHorizontalStrut(4));
		_Hfields.add(_Hslider);
		_Hfields.add(Box.createHorizontalStrut(2));
		_Hfields.add(_Hhint);
		_Hfields.add(Box.createHorizontalStrut(4));

		_Sfields = Box.createHorizontalBox();

		_Slabel = new JLabel("彩度:0   ");
		_Sslider = new JSlider(-128, 128, 0);
		_Sslider.addChangeListener(this);

		_Shint = new JLabel("?");
		_Shint.setToolTipText("各ピクセルの彩度に加算する値を設定");

		_Sfields.add(Box.createHorizontalStrut(4));
		_Sfields.add(_Slabel);
		_Sfields.add(Box.createHorizontalStrut(4));
		_Sfields.add(_Sslider);
		_Sfields.add(Box.createHorizontalStrut(2));
		_Sfields.add(_Shint);
		_Sfields.add(Box.createHorizontalStrut(4));

		_Bfields = Box.createHorizontalBox();

		_Blabel = new JLabel("明度:0   ");
		_Bslider = new JSlider(-128, 128, 0);
		_Bslider.addChangeListener(this);

		_Bhint = new JLabel("?");
		_Bhint.setToolTipText("各ピクセルの明度に加算する値を設定");

		_Bfields.add(Box.createHorizontalStrut(4));
		_Bfields.add(_Blabel);
		_Bfields.add(Box.createHorizontalStrut(4));
		_Bfields.add(_Bslider);
		_Bfields.add(Box.createHorizontalStrut(2));
		_Bfields.add(_Bhint);
		_Bfields.add(Box.createHorizontalStrut(4));

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_views);
		c.add(Box.createVerticalStrut(4));
		c.add(_Hfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Sfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Bfields);
		c.add(Box.createVerticalStrut(8));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_h = 999;
		_s = 999;
		_b = 999;

	}

	public int getH() {
		return _h;
	}

	public int getS() {
		return _s;
	}

	public int getB() {
		return _b;
	}

	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == _btnExec) {

			_h = _Hslider.getValue();
			_s = _Sslider.getValue();
			_b = _Bslider.getValue();

			hide();

		}

		if (e.getSource() == _btnCancel) {

			_h = 999;
			_s = 999;
			_b = 999;

			hide();

		}

	}

	public void stateChanged(ChangeEvent e) {

		int dh = _Hslider.getValue();
		int ds = _Sslider.getValue();
		int db = _Bslider.getValue();

		// 新しい設定でプレビュー画像を再作成
		_preview.assign(_original);
		_preview.addHSB(dh, ds, db);

		// プレビュー画像をパネルに表示
		_viewP.setPixels(_preview.getPixels());

		repaint();

		if (e.getSource() == _Hslider) {

			int val = _Hslider.getValue();

			_Hlabel.setText("色相:" + String.valueOf(val));

		}

		if (e.getSource() == _Sslider) {

			int val = _Sslider.getValue();

			_Slabel.setText("彩度:" + String.valueOf(val));

		}

		if (e.getSource() == _Bslider) {

			int val = _Bslider.getValue();

			_Blabel.setText("明度:" + String.valueOf(val));

		}

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
