/**
	RGB成分調整ダイアログ
*/
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class RGBCondDialog extends JDialog implements ActionListener, ChangeListener, ComponentListener {

	private gp10j _owner;

	private Box _views, _Rfields, _Gfields, _Bfields, _buttons;

	private PixelsViewer _viewO, _viewP;
	private JLabel _viewLabel;

	private JLabel _Rlabel, _Glabel, _Blabel;
	private JSlider _Rslider, _Gslider, _Bslider;
	private JLabel _Rhint, _Ghint, _Bhint;

	private JButton _btnExec, _btnCancel;

	private int _r, _g, _b;

	private int[] _original;
	private JIntFrameBuffer _preview;

	public RGBCondDialog(gp10j owner, JIntFrameBuffer buf) {

		super(owner, "RGB成分調整", true);

		_owner = owner;

		int w = buf.getWidth();
		int h = buf.getHeight();

		int width = 0;
		int height = 0;

		// プレビュー画像のサイズ算出
		if (w > h) {

			if (w < gp10j.PREVIEW_SIZE) {
				width = w;
			} else {
				width = gp10j.PREVIEW_SIZE;
			}

			height = (h * width) / w;

		} else {

			if (h < gp10j.PREVIEW_SIZE) {
				height = h;
			} else {
				height = gp10j.PREVIEW_SIZE;
			}

			width = (w * height) / h;

		}

		// プレビュー画像のピクセル列作成
		_original = buf.getScaledPixels(width, height);

		// プレビュー画像作成
		_preview = new JIntFrameBuffer(_original);

		_views = Box.createHorizontalBox();

		_viewO = new PixelsViewer(_original);
		_viewLabel = new JLabel("→");
		_viewP = new PixelsViewer(_original);

		_views.add(_viewO);
		_views.add(_viewLabel);
		_views.add(_viewP);

		_Rfields = Box.createHorizontalBox();

		_Rlabel = new JLabel("R:0   ");
		_Rslider = new JSlider(-128, 128, 0);
		_Rslider.addChangeListener(this);
		_Rhint = new JLabel("?");
		_Rhint.setToolTipText("各ピクセルの赤成分に加算する値を設定");

		_Rfields.add(Box.createHorizontalStrut(4));
		_Rfields.add(_Rlabel);
		_Rfields.add(Box.createHorizontalStrut(4));
		_Rfields.add(_Rslider);
		_Rfields.add(Box.createHorizontalStrut(1));
		_Rfields.add(_Rhint);
		_Rfields.add(Box.createHorizontalStrut(2));

		_Gfields = Box.createHorizontalBox();

		_Glabel = new JLabel("G:0   ");
		_Gslider = new JSlider(-128, 128, 0);
		_Gslider.addChangeListener(this);
		_Ghint = new JLabel("?");
		_Ghint.setToolTipText("各ピクセルの緑成分に加算する値を設定");

		_Gfields.add(Box.createHorizontalStrut(4));
		_Gfields.add(_Glabel);
		_Gfields.add(Box.createHorizontalStrut(4));
		_Gfields.add(_Gslider);
		_Gfields.add(Box.createHorizontalStrut(1));
		_Gfields.add(_Ghint);
		_Gfields.add(Box.createHorizontalStrut(2));

		_Bfields = Box.createHorizontalBox();

		_Blabel = new JLabel("B:0   ");
		_Bslider = new JSlider(-128, 128, 0);
		_Bslider.addChangeListener(this);
		_Bhint = new JLabel("?");
		_Bhint.setToolTipText("各ピクセルの青成分に加算する値を設定");

		_Bfields.add(Box.createHorizontalStrut(4));
		_Bfields.add(_Blabel);
		_Bfields.add(Box.createHorizontalStrut(4));
		_Bfields.add(_Bslider);
		_Bfields.add(Box.createHorizontalStrut(1));
		_Bfields.add(_Bhint);
		_Bfields.add(Box.createHorizontalStrut(2));

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_views);
		c.add(Box.createVerticalStrut(4));
		c.add(_Rfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Gfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Bfields);
		c.add(Box.createVerticalStrut(8));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_r = 999;
		_g = 999;
		_b = 999;

		addComponentListener(this);

	}

	public int getR() {
		return _r;
	}

	public int getG() {
		return _g;
	}

	public int getB() {
		return _b;
	}

	public void actionPerformed(ActionEvent e) {

		// 「実行」ボタン
		if (e.getSource() == _btnExec) {

			_r = _Rslider.getValue();
			_g = _Gslider.getValue();
			_b = _Bslider.getValue();

			hide();

		}

		// 「中止」ボタン
		if (e.getSource() == _btnCancel) {

			_r = 999;
			_g = 999;
			_b = 999;

			hide();

		}

	}

	// スライダー値変更時の処理
	public void stateChanged(ChangeEvent e) {

		int dr = _Rslider.getValue();
		int dg = _Gslider.getValue();
		int db = _Bslider.getValue();

		// 新しい設定でプレビュー画像を再作成
		_preview.assign(_original);
		_preview.addRGB(dr, dg, db);

		// プレビュー画像をパネルに表示
		_viewP.setPixels(_preview.getPixels());

		repaint();

		// Rスライダー値変更
		if (e.getSource() == _Rslider) {

			int val = _Rslider.getValue();

			_Rlabel.setText("R:" + String.valueOf(val));

		}

		// Gスライダー値変更
		if (e.getSource() == _Gslider) {

			int val = _Gslider.getValue();

			_Glabel.setText("G:" + String.valueOf(val));

		}

		// Bスライダー値変更
		if (e.getSource() == _Bslider) {

			int val = _Bslider.getValue();

			_Blabel.setText("B:" + String.valueOf(val));

		}

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
