import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class ResizeDialog extends JDialog implements ActionListener, DocumentListener, ComponentListener {

	private gp10j _owner;

	private JTextField _widthField, _heightField;
	private JCheckBox _chk;

	private Box _fields, _buttons;
	private JButton _btnExec, _btnCancel;

	private int _originalWidth, _originalHeight;
	private int _width, _height;

	public ResizeDialog(gp10j owner) {

		super(owner, "サイズ変更", true);

		_owner = owner;
		addComponentListener(this);

		_fields = Box.createHorizontalBox();

		_widthField = new JTextField(8);
		_widthField.getDocument().addDocumentListener(this);

		_heightField = new JTextField(8);
		_heightField.getDocument().addDocumentListener(this);

		_chk = new JCheckBox("縦横比維持", true);

		// サイズ入力欄作成
		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(new JLabel("横幅"));
		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(_widthField);
		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(new JLabel("高さ"));
		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(_heightField);
		_fields.add(Box.createHorizontalStrut(4));
		_fields.add(_chk);
		_fields.add(Box.createHorizontalStrut(4));

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_fields.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_fields);
		c.add(Box.createVerticalStrut(8));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_width = -1;
		_height = -1;

	}

	public void setOriginalSize(int width, int height) {

		_originalWidth = width;
		_originalHeight = height;

		_widthField.setText(String.valueOf(width));
		_heightField.setText(String.valueOf(height));

	}

	public Dimension getImageSize() {

		if (_width < 1 || _height < 1) {
			return null;
		}

		Dimension d = new Dimension(_width, _height);

		return d;

	}

	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == _btnExec) {

			try {
				_width = Integer.parseInt(_widthField.getText());
			} catch (Exception ex) {
				_width = 0;
			}

			try {
				_height = Integer.parseInt(_heightField.getText());
			} catch (Exception ex) {
				_height = 0;
			}

			hide();

		}

		if (e.getSource() == _btnCancel) {

			_width = -1;
			_height = -1;

			hide();

		}

	}

	public void insertUpdate(DocumentEvent e) {
		fieldUpdate(e);
	}

	public void removeUpdate(DocumentEvent e) {
		fieldUpdate(e);
	}

	public void changedUpdate(DocumentEvent e) {
		fieldUpdate(e);
	}

	private void fieldUpdate(DocumentEvent e) {

		if (e.getDocument() == _widthField.getDocument()) {

			int width = 0;

			try {
				width = Integer.parseInt(_widthField.getText());
			} catch (Exception ex) {
				width = 0;
			}

			if (width > 0 && _chk.isSelected()) {

				int height = (width * _originalHeight) / _originalWidth;

				if (height < 0) {
					height = 1;
				}

				_heightField.setText(String.valueOf(height));

			}

		}

		if (e.getDocument() == _heightField.getDocument()) {


		}

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
