import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class TrimDialog extends JDialog implements ActionListener, DocumentListener, ComponentListener {

	private gp10j _owner;

	private JTextField _xField, _yField;
	private JTextField _widthField, _heightField;

	private Box _fieldsXY, _fieldsSize, _buttons;
	private JButton _btnExec, _btnCancel;

	private int _trimX, _trimY;
	private int _trimWidth, _trimHeight;

	public TrimDialog(gp10j owner) {

		super(owner, "切り出し", true);

		_owner = owner;
		addComponentListener(this);

		_fieldsXY = Box.createHorizontalBox();

		_xField = new JTextField(8);
		_xField.getDocument().addDocumentListener(this);

		_yField = new JTextField(8);
		_yField.getDocument().addDocumentListener(this);

		// サイズ入力欄作成
		_fieldsXY.add(Box.createHorizontalStrut(4));
		_fieldsXY.add(new JLabel("X"));
		_fieldsXY.add(Box.createHorizontalStrut(4));
		_fieldsXY.add(_xField);
		_fieldsXY.add(Box.createHorizontalStrut(4));
		_fieldsXY.add(new JLabel("Y"));
		_fieldsXY.add(Box.createHorizontalStrut(4));
		_fieldsXY.add(_yField);

		_fieldsSize = Box.createHorizontalBox();

		_widthField = new JTextField(8);
		_widthField.getDocument().addDocumentListener(this);

		_heightField = new JTextField(8);
		_heightField.getDocument().addDocumentListener(this);

		// サイズ入力欄作成
		_fieldsSize.add(Box.createHorizontalStrut(4));
		_fieldsSize.add(new JLabel("横幅"));
		_fieldsSize.add(Box.createHorizontalStrut(4));
		_fieldsSize.add(_widthField);
		_fieldsSize.add(Box.createHorizontalStrut(4));
		_fieldsSize.add(new JLabel("高さ"));
		_fieldsSize.add(Box.createHorizontalStrut(4));
		_fieldsSize.add(_heightField);
		_fieldsSize.add(Box.createHorizontalStrut(4));

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
//		_fieldsSize.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_fieldsXY);
		c.add(Box.createVerticalStrut(4));
		c.add(_fieldsSize);
		c.add(Box.createVerticalStrut(8));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_trimWidth = -1;
		_trimHeight = -1;

	}

	public void setOriginalSize(int width, int height) {

		_widthField.setText(String.valueOf(width));
		_heightField.setText(String.valueOf(height));

	}

	public int getTrimX() {
		return _trimX;
	}

	public int getTrimY() {
		return _trimY;
	}

	public int getTrimWidth() {
		return _trimWidth;
	}

	public int getTrimHeight() {
		return _trimHeight;
	}

	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == _btnExec) {

			try {
				_trimX = Integer.parseInt(_xField.getText());
			} catch (Exception ex) {
				_trimX = 0;
			}

			try {
				_trimY = Integer.parseInt(_yField.getText());
			} catch (Exception ex) {
				_trimY = 0;
			}

			try {
				_trimWidth = Integer.parseInt(_widthField.getText());
			} catch (Exception ex) {
				_trimWidth = 0;
			}

			try {
				_trimHeight = Integer.parseInt(_heightField.getText());
			} catch (Exception ex) {
				_trimHeight = 0;
			}

			_owner.setTrimArea(0, 0, 0, 0);

			hide();

		}

		if (e.getSource() == _btnCancel) {

			_trimWidth = -1;
			_trimHeight = -1;

			_owner.setTrimArea(0, 0, 0, 0);

			hide();

		}

	}

	public void insertUpdate(DocumentEvent e) {
		fieldUpdate(e);
	}

	public void removeUpdate(DocumentEvent e) {
		fieldUpdate(e);
	}

	public void changedUpdate(DocumentEvent e) {
		fieldUpdate(e);
	}

	private void fieldUpdate(DocumentEvent e) {

		int x = 0, y = 0, width = 0, height = 0;

		try {
			x = Integer.parseInt(_xField.getText());
		} catch (Exception ex1) {
			x = 0;
		}

		try {
			y = Integer.parseInt(_yField.getText());
		} catch (Exception ex2) {
			y = 0;
		}

		try {
			width = Integer.parseInt(_widthField.getText());
		} catch (Exception ex3) {
			width = 0;
		}

		try {
			height = Integer.parseInt(_heightField.getText());
		} catch (Exception ex4) {
			height = 0;
		}

		_owner.setTrimArea(x, y, width, height);

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
