import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class UnsharpDialog extends JDialog implements ActionListener, ChangeListener, ComponentListener {

	private gp10j _owner;

	private PixelsViewer _viewO, _viewP;
	private JLabel _viewLabel;

	private Box _views, _Rfields, _Vfields, _buttons;

	private JLabel _Rlabel, _Vlabel;
	private JSlider _Rslider, _Vslider;
	private JLabel _Rhint, _Vhint;

	private JButton _btnExec, _btnPreview, _btnCancel;

	private int _range, _val;

	private int[] _original;
	private JIntFrameBuffer _preview;

	public UnsharpDialog(gp10j owner, JIntFrameBuffer buf) {

		super(owner, "アンシャープマスク", true);

		_owner = owner;

		int w = buf.getWidth();
		int h = buf.getHeight();

		int width = 0;
		int height = 0;

		if (w < gp10j.PREVIEW_SIZE) {
			width = w;
		} else {
			width = gp10j.PREVIEW_SIZE;
		}

		if (h < gp10j.PREVIEW_SIZE) {
			height = h;
		} else {
			height = gp10j.PREVIEW_SIZE;
		}

		_views = Box.createHorizontalBox();

		_original = buf.getSubPixels(0, 0, width, height);
		_preview = new JIntFrameBuffer(_original);

		_viewO = new PixelsViewer(_original);
		_viewLabel = new JLabel("→");
		_viewP = new PixelsViewer(_original);

		_views.add(_viewO);
		_views.add(_viewLabel);
		_views.add(_viewP);

		_Rfields = Box.createHorizontalBox();

		_Rlabel = new JLabel("範囲:1");
		_Rslider = new JSlider(1, 5, 1);
		_Rslider.addChangeListener(this);

		_Rhint = new JLabel("?");
		_Rhint.setToolTipText("フィルタ処理時の計算範囲。大きくすると、処理に時間がかかります。");

		_Rfields.add(Box.createHorizontalStrut(4));
		_Rfields.add(_Rlabel);
		_Rfields.add(Box.createHorizontalStrut(4));
		_Rfields.add(_Rslider);
		_Rfields.add(Box.createHorizontalStrut(1));
		_Rfields.add(_Rhint);
		_Rfields.add(Box.createHorizontalStrut(2));

		_Vfields = Box.createHorizontalBox();

		_Vlabel = new JLabel("強度:1.0");
		_Vslider = new JSlider(-40, 40, 10);
		_Vslider.addChangeListener(this);

		_Vhint = new JLabel("?");
		_Vhint.setToolTipText("フィルタの強度。負の値でぼかし、正の値で先鋭化の効果が得られます。");

		_Vfields.add(Box.createHorizontalStrut(4));
		_Vfields.add(_Vlabel);
		_Vfields.add(Box.createHorizontalStrut(4));
		_Vfields.add(_Vslider);
		_Vfields.add(Box.createHorizontalStrut(1));
		_Vfields.add(_Vhint);
		_Vfields.add(Box.createHorizontalStrut(2));

		_buttons = Box.createHorizontalBox();

		_btnExec = new JButton("実行");
		_btnExec.addActionListener(this);

		_btnPreview = new JButton("確認");
		_btnPreview.addActionListener(this);

		_btnCancel = new JButton("中止");
		_btnCancel.addActionListener(this);

		// ボタン欄作成
		_buttons.add(_btnExec);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnPreview);
		_buttons.add(Box.createHorizontalStrut(4));
		_buttons.add(_btnCancel);

		Container c = getContentPane();

		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));

		c.add(Box.createVerticalStrut(4));
		c.add(_views);
		c.add(Box.createVerticalStrut(4));
		c.add(_Rfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_Vfields);
		c.add(Box.createVerticalStrut(4));
		c.add(_buttons);
		c.add(Box.createVerticalStrut(4));

		setResizable(false);
		pack();

		// アプリケーションの位置取得
		Rectangle rec = owner.getBounds(null);

		// アプリケーション中央座標算出
		int appCenterX = rec.x + rec.width / 2;
		int appCenterY = rec.y + rec.height / 2;

		// ダイアログをアプリケーション中央座標にあわせて配置
		setLocation(appCenterX - getWidth() / 2, appCenterY - getHeight() / 2);

		_range = -1;

		addComponentListener(this);

		repaint();

	}

	public int getRange() {
		return _range;
	}

	public int getVal() {
		return _val;
	}

	public void actionPerformed(ActionEvent e) {

		// 「実行」ボタン
		if (e.getSource() == _btnExec) {

			_range = _Rslider.getValue();
			_val = _Vslider.getValue();

			hide();

		}

		// 「確認」ボタン
		if (e.getSource() == _btnPreview) {

			int range = _Rslider.getValue();
			double val = (double)(_Vslider.getValue()) / 10.0;

			_preview.assign(_original);

			_preview.unsharp(range, val);

			_viewP.setPixels(_preview.getPixels());

			repaint();

		}

		// 「中止」ボタン
		if (e.getSource() == _btnCancel) {

			_range = -1;

			hide();

		}

	}

	public void stateChanged(ChangeEvent e) {

		int range = _Rslider.getValue();
		double val = (double)(_Vslider.getValue()) / 10.0;

		_Rlabel.setText("範囲:" + range);
		_Vlabel.setText("強度:" + val);

		repaint();

	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		_owner.repaint();
	}

	public void componentResized(ComponentEvent e) {
	}

	public void componentShown(ComponentEvent e) {
	}

}
